import QtQuick 2.0;
import MuseScore 3.0;
import QtQuick.Controls 1.0;
import QtQuick.Controls.Styles 1.3;
import QtQuick.Dialogs 1.1;

MuseScore
{
	menuPath: "Plugins.TuningCents";
	description: "This plugin applies natural tunings, e.g. Pythagorian, on the selected notes of the score, or the whole score if no notes are selected.";
	version: "1.0";
	pluginType: "dialog";
	width: 356;
	height: 235;

	property var allKeys: [theKeyboard.theC, theKeyboard.theDb, theKeyboard.theD, theKeyboard.theEb, theKeyboard.theE, theKeyboard.theF, theKeyboard.theGb, theKeyboard.theG, theKeyboard.theAb, theKeyboard.theA, theKeyboard.theBb, theKeyboard.theB];
	property var naturalTuning: [ 0, -10,  4, -6, -16, -2, -12, 2, -8,  6,  -4, -14 ];
	
	Item
	{
		id: tuningCents; width: 356; height: 235;
		
		Grid
		{
			rows: 2; columns: 1; spacing: 4;
			
			Grid
			{
				rows: 1; columns: 5; spacing: 5;
				
				Label
				{
					text: "Base\nPitch";
				}
				
				ComboBox
				{
					id: theBase; width: 80;
					model: ["C", "C#/Db", "D", "D#/Eb", "E", "F", "F#/Gb", "G", "G#/Ab", "A", "A#/Bb", "B"]
				}
				
				Button
				{
					text: "Natural";
					onClicked: {for(var i=0; i<allKeys.length; i++) allKeys[i].centsValue = naturalTuning[(i+1 - (theBase.currentIndex+1)%12 + 12) % 12];}
				}
				
				Button
				{
					text: "Tempered";
					onClicked: { for(var i=0; i<allKeys.length; i++) allKeys[i].centsValue = 0; }
				}
				
				Button
				{
					text: "Apply";
					onClicked: { if (isFullScore()==true) msgDialog.open(); else processSelectedNotes(); }
				}
			}
			
			Keyboard {id: theKeyboard;}
		}
	}
	
	function isFullScore()
	{
		var cursor = curScore.newCursor();
		cursor.rewind(1);
		
		if (!cursor.segment) //nothing selected
			return true;
		else
			return false;
	}
	
	MessageDialog
	{
		id: msgDialog
		title: "Process Full Score";
		text: "Nothing is selected. Process full score?";
		standardButtons: StandardButton.Yes | StandardButton.Abort;
		onYes: { processSelectedNotes(); }
		onRejected: { return; }
	}

	function processSelectedNotes()
	{
		var acc = []
		
		var cursor = curScore.newCursor(); cursor.rewind(1);
		var startStaff;
		var endStaff;
		var endTick;
		var fullScore = false;
//
		if (!cursor.segment) //nothing selected
		{
			fullScore = true;
			startStaff = 0;
			endStaff = curScore.nstaves - 1;
		}
		else
		{
			startStaff = cursor.staffIdx;
			cursor.rewind(2); //end of selection
			if (cursor.tick == 0) { endTick = curScore.lastSegment.tick+1; }
			else { endTick = cursor.tick; }
			endStaff = cursor.staffIdx;
		}
//
		for (var staff = startStaff; staff <= endStaff; staff++)
		{
			for (var voice = 0; voice < 4; voice++)
			{
				cursor.rewind(1);
				cursor.voice = voice;
				cursor.staffIdx = staff;
				
				if (fullScore) cursor.rewind(0);
				var measure = cursor.measure;
				while (cursor.segment && (fullScore || cursor.tick < endTick))
				{
					if (cursor.measure != measure)
					{
						measure = cursor.measure;
						for (var i=0;i<acc.length;i++) acc.pop();
					}
					if (cursor.element && cursor.element.type == Ms.CHORD )
					{
						var notes = cursor.element.notes;
						for (var i=0; i< notes.length; i++)
						{
							var note = notes[i];
							note.tuning = allKeys[note.pitch%12].centsValue;
						}
					}
					cursor.next();
				} //while segments available
			} //for all voices
		} //for all staves
	}
	
	onRun:
	{
		if (typeof curScore === 'undefined') Qt.quit();
	}
}
